﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Threading;

namespace AppDomainPerformanceDemo
{
    public static class CodeTimer
    {
        [DllImport( "kernel32.dll", SetLastError = true )]
        static extern bool GetThreadTimes( IntPtr hThread, out long lpCreationTime,
           out long lpExitTime, out long lpKernelTime, out long lpUserTime );

        [DllImport( "kernel32.dll" )]
        static extern IntPtr GetCurrentThread();

        private static long GetCurrentThreadTimes()
        {
            long l;
            long kernelTime, userTimer;
            GetThreadTimes( GetCurrentThread(), out l, out l,
                out kernelTime, out userTimer );
            return kernelTime + userTimer;
        }

        static CodeTimer()
        {
            Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.High;
            Thread.CurrentThread.Priority = ThreadPriority.Highest;

        }

        public static void Time( string name, int iteration, Action action )
        {
            if( String.IsNullOrEmpty( name ) )
            {
                return;
            }

            if( action == null )
            {
                return;
            }

            // 1.
            ConsoleColor currentForeColor = Console.ForegroundColor;
            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.WriteLine( name );

            // 2.
            GC.Collect( GC.MaxGeneration, GCCollectionMode.Forced );
            int[] gcCounts = new int[GC.MaxGeneration + 1];
            for( int i = 0; i <= GC.MaxGeneration; i++ )
            {
                gcCounts[i] = GC.CollectionCount( i );
            }

            // 3.
            Stopwatch watch = new Stopwatch();
            watch.Start();
            long ticksFst = GetCurrentThreadTimes(); //100 nanosecond one tick

            for( int i = 0; i < iteration; i++ ) action();
            long ticks = GetCurrentThreadTimes() - ticksFst;
            watch.Stop();

            // 4.
            Console.ForegroundColor = currentForeColor;
            Console.WriteLine( "\tTime Elapsed:\t" +
               watch.ElapsedMilliseconds.ToString( "N0" ) + "ms" );

            Console.WriteLine( "\tCall:        \t" +
                (iteration/watch.ElapsedMilliseconds)*1000 + "/s" );


            Console.WriteLine( "\tCPU time:\t" + ticks.ToString( "N0" ) + "ns" );

            // 5.
            for( int i = 0; i <= GC.MaxGeneration; i++ )
            {
                int count = GC.CollectionCount( i ) - gcCounts[i];
                Console.WriteLine( "\tGen " + i + ": \t\t" + count );
            }

            Console.WriteLine();

        }
    }

}
